package routines;

/*
 * user specification: the function's comment should contain keys as follows: 1. write about the function's comment.but
 * it must be before the "{talendTypes}" key.
 * 
 * 2. {talendTypes} 's value must be talend Type, it is required . its value should be one of: String, char | Character,
 * long | Long, int | Integer, boolean | Boolean, byte | Byte, Date, double | Double, float | Float, Object, short |
 * Short
 * 
 * 3. {Category} define a category for the Function. it is required. its value is user-defined .
 * 
 * 4. {param} 's format is: {param} <type>[(<default value or closed list values>)] <name>[ : <comment>]
 * 
 * <type> 's value should be one of: string, int, list, double, object, boolean, long, char, date. <name>'s value is the
 * Function's parameter name. the {param} is optional. so if you the Function without the parameters. the {param} don't
 * added. you can have many parameters for the Function.
 * 
 * 5. {example} gives a example for the Function. it is optional.
 */
public class MD4 {

	int A, B, C, D;
	int d[];
	int numwords;

	/*
	 * For verification of a modicum of sanity, run a few test strings through
	 */
	
	MD4(String s) {
		byte in[] = new byte[s.length()];
		int i;

		for (i = 0; i < s.length(); i++) {
			in[i] = (byte) (s.charAt(i) & 0xff);
		}
		mdinit(in);
	}

	MD4(byte in[]) {
		mdinit(in);
	}

	void mdinit(byte in[]) {
		int newlen, endblklen, pad, i;
		long datalenbits;

		datalenbits = in.length * 8;
		endblklen = in.length % 64;
		if (endblklen < 56) {
			pad = 64 - endblklen;
		} else {
			pad = (64 - endblklen) + 64;
		}
		newlen = in.length + pad;
		byte b[] = new byte[newlen];
		for (i = 0; i < in.length; i++) {
			b[i] = in[i];
		}
		b[in.length] = (byte) 0x80;
		for (i = b.length + 1; i < (newlen - 8); i++) {
			b[i] = 0;
		}
		for (i = 0; i < 8; i++) {
			b[newlen - 8 + i] = (byte) (datalenbits & 0xff);
			datalenbits >>= 8;
		}
		/* init registers */
		A = 0x67452301;
		B = 0xefcdab89;
		C = 0x98badcfe;
		D = 0x10325476;
		this.numwords = newlen / 4;
		this.d = new int[this.numwords];
		for (i = 0; i < newlen; i += 4) {
			this.d[i / 4] = (b[i] & 0xff) + ((b[i + 1] & 0xff) << 8) + ((b[i + 2] & 0xff) << 16)
					+ ((b[i + 3] & 0xff) << 24);
		}
	}

	public String toString() {
		String s;

		return (tohex(A) + tohex(B) + tohex(C) + tohex(D));
	}

	int[] getregs() {
		int regs[] = { this.A, this.B, this.C, this.D };

		return regs;
	}

	void calc() {
		int AA, BB, CC, DD, i;

		for (i = 0; i < numwords / 16; i++) {
			AA = A;
			BB = B;
			CC = C;
			DD = D;
			round1(i);
			round2(i);
			round3(i);
			A += AA;
			B += BB;
			C += CC;
			D += DD;
		}
	}

	static int F(int x, int y, int z) {
		return ((x & y) | (~x & z));
	}

	static int G(int x, int y, int z) {
		return ((x & y) | (x & z) | (y & z));
	}

	static int H(int x, int y, int z) {
		return (x ^ y ^ z);
	}

	void round1(int blk) {
		A = rotintlft((A + F(B, C, D) + d[0 + 16 * blk]), 3);
		D = rotintlft((D + F(A, B, C) + d[1 + 16 * blk]), 7);
		C = rotintlft((C + F(D, A, B) + d[2 + 16 * blk]), 11);
		B = rotintlft((B + F(C, D, A) + d[3 + 16 * blk]), 19);

		A = rotintlft((A + F(B, C, D) + d[4 + 16 * blk]), 3);
		D = rotintlft((D + F(A, B, C) + d[5 + 16 * blk]), 7);
		C = rotintlft((C + F(D, A, B) + d[6 + 16 * blk]), 11);
		B = rotintlft((B + F(C, D, A) + d[7 + 16 * blk]), 19);

		A = rotintlft((A + F(B, C, D) + d[8 + 16 * blk]), 3);
		D = rotintlft((D + F(A, B, C) + d[9 + 16 * blk]), 7);
		C = rotintlft((C + F(D, A, B) + d[10 + 16 * blk]), 11);
		B = rotintlft((B + F(C, D, A) + d[11 + 16 * blk]), 19);

		A = rotintlft((A + F(B, C, D) + d[12 + 16 * blk]), 3);
		D = rotintlft((D + F(A, B, C) + d[13 + 16 * blk]), 7);
		C = rotintlft((C + F(D, A, B) + d[14 + 16 * blk]), 11);
		B = rotintlft((B + F(C, D, A) + d[15 + 16 * blk]), 19);
	}

	void round2(int blk) {
		A = rotintlft((A + G(B, C, D) + d[0 + 16 * blk] + 0x5a827999), 3);
		D = rotintlft((D + G(A, B, C) + d[4 + 16 * blk] + 0x5a827999), 5);
		C = rotintlft((C + G(D, A, B) + d[8 + 16 * blk] + 0x5a827999), 9);
		B = rotintlft((B + G(C, D, A) + d[12 + 16 * blk] + 0x5a827999), 13);

		A = rotintlft((A + G(B, C, D) + d[1 + 16 * blk] + 0x5a827999), 3);
		D = rotintlft((D + G(A, B, C) + d[5 + 16 * blk] + 0x5a827999), 5);
		C = rotintlft((C + G(D, A, B) + d[9 + 16 * blk] + 0x5a827999), 9);
		B = rotintlft((B + G(C, D, A) + d[13 + 16 * blk] + 0x5a827999), 13);

		A = rotintlft((A + G(B, C, D) + d[2 + 16 * blk] + 0x5a827999), 3);
		D = rotintlft((D + G(A, B, C) + d[6 + 16 * blk] + 0x5a827999), 5);
		C = rotintlft((C + G(D, A, B) + d[10 + 16 * blk] + 0x5a827999), 9);
		B = rotintlft((B + G(C, D, A) + d[14 + 16 * blk] + 0x5a827999), 13);

		A = rotintlft((A + G(B, C, D) + d[3 + 16 * blk] + 0x5a827999), 3);
		D = rotintlft((D + G(A, B, C) + d[7 + 16 * blk] + 0x5a827999), 5);
		C = rotintlft((C + G(D, A, B) + d[11 + 16 * blk] + 0x5a827999), 9);
		B = rotintlft((B + G(C, D, A) + d[15 + 16 * blk] + 0x5a827999), 13);

	}

	void round3(int blk) {
		A = rotintlft((A + H(B, C, D) + d[0 + 16 * blk] + 0x6ed9eba1), 3);
		D = rotintlft((D + H(A, B, C) + d[8 + 16 * blk] + 0x6ed9eba1), 9);
		C = rotintlft((C + H(D, A, B) + d[4 + 16 * blk] + 0x6ed9eba1), 11);
		B = rotintlft((B + H(C, D, A) + d[12 + 16 * blk] + 0x6ed9eba1), 15);

		A = rotintlft((A + H(B, C, D) + d[2 + 16 * blk] + 0x6ed9eba1), 3);
		D = rotintlft((D + H(A, B, C) + d[10 + 16 * blk] + 0x6ed9eba1), 9);
		C = rotintlft((C + H(D, A, B) + d[6 + 16 * blk] + 0x6ed9eba1), 11);
		B = rotintlft((B + H(C, D, A) + d[14 + 16 * blk] + 0x6ed9eba1), 15);

		A = rotintlft((A + H(B, C, D) + d[1 + 16 * blk] + 0x6ed9eba1), 3);
		D = rotintlft((D + H(A, B, C) + d[9 + 16 * blk] + 0x6ed9eba1), 9);
		C = rotintlft((C + H(D, A, B) + d[5 + 16 * blk] + 0x6ed9eba1), 11);
		B = rotintlft((B + H(C, D, A) + d[13 + 16 * blk] + 0x6ed9eba1), 15);

		A = rotintlft((A + H(B, C, D) + d[3 + 16 * blk] + 0x6ed9eba1), 3);
		D = rotintlft((D + H(A, B, C) + d[11 + 16 * blk] + 0x6ed9eba1), 9);
		C = rotintlft((C + H(D, A, B) + d[7 + 16 * blk] + 0x6ed9eba1), 11);
		B = rotintlft((B + H(C, D, A) + d[15 + 16 * blk] + 0x6ed9eba1), 15);

	}

	void round4(int blk) {
		System.out.println(" must be md5, in round4!");
	}
	
	static int rotintlft(int val, int numbits) {
		return ((val << numbits) | (val >>> (32 - numbits)));
	}

	static String tohex(int i) {
		int b;
		String tmpstr;

		tmpstr = "";
		for (b = 0; b < 4; b++) {
			tmpstr += Integer.toString((i >> 4) & 0xf, 16) + Integer.toString(i & 0xf, 16);
			i >>= 8;
		}
		return tmpstr;
	}


}
